@@ -18,7 +18,7 @@ limitations under the License.
 """
 
 PACKAGE_NAME = "arcana"
-CODE_URL = f"https://github.com/australian-imaging-service/{PACKAGE_NAME}"
+CODE_URL = f"https://github.com/ArcanaFramework/{PACKAGE_NAME}"
 
 __authors__ = [("Thomas G. Close", "tom.g.close@gmail.com")]
 
@@ -26,7 +26,7 @@ from ._version import __version__
 
 from pydra import set_input_validator
 
-set_input_validator(True)
+# set_input_validator(True)
 # from .core.data.set import Dataset
 # from .core.data.store import DataStore
 
