@@ -2,6 +2,9 @@ from __future__ import annotations
 import subprocess as sp
 import typing as ty
 import re
+import traceback
+import yaml
+import difflib
 from itertools import zip_longest
 from pathlib import Path, PosixPath
 import tempfile
@@ -9,13 +12,14 @@ import tarfile
 import logging
 import docker
 import os.path
+import attrs
 from contextlib import contextmanager
 from collections.abc import Iterable
 import cloudpickle as cp
 from pydra.engine.core import Workflow, LazyField, TaskBase
 from pydra.engine.task import FunctionTask
 from pydra.engine.specs import BaseSpec, SpecInfo
-from arcana.core.exceptions import ArcanaUsageError
+from arcana.core.exceptions import ArcanaUsageError, ArcanaError
 
 
 logger = logging.getLogger("arcana")
@@ -40,6 +44,48 @@ ARCANA_PIP = "git+ssh://git@github.com/australian-imaging-service/arcana.git"
 HASH_CHUNK_SIZE = 2**20  # 1MB in calc. checksums to avoid mem. issues
 
 
+@attrs.define
+class NestedContext:
+    """Base class for "nested contexts", which can be used in "with" statements at
+    at multiple points in the API, and ensures that the context is only entered at most
+    once at any one point. This allows low level calls to ensure that they are executing
+    within an appropriate context, while also enabling high level calls to maintain a
+    context over multiple low-level calls, and thereby not take the performance hit of
+    continually setting up and breaking down the context.
+
+    Parameters
+    -----------
+    _type_
+        _description_
+    """
+
+    depth: int = attrs.field(default=0, init=False)
+
+    def __enter__(self):
+        # This allows the store to be used within nested contexts
+        # but still only use one connection. This is useful for calling
+        # methods that need connections, and therefore control their
+        # own connection, in batches using the same connection by
+        # placing the batch calls within an outer context.
+        self.depth += 1
+        if self.depth == 1:
+            self.enter()
+        return self
+
+    def __exit__(self, exception_type, exception_value, traceback):
+        self.depth -= 1
+        if self.depth == 0:
+            self.exit()
+
+    def enter(self):
+        "To be overridden in subclasses as necessary"
+        pass
+
+    def exit(self):
+        "To be overridden in subclasses as necessary"
+        pass
+
+
 def get_home_dir():
     try:
         home_dir = Path(os.environ["ARCANA_HOME"])
@@ -497,27 +543,38 @@ def extract_file_from_docker_image(
     try:
         dc.api.pull(image_tag)
     except docker.errors.APIError as e:
-        if e.response.status_code in (404, 500):
+        if e.response.status_code == 404:
+            return None
+        elif e.response.status_code == 500:
+            logger.warning(
+                "Could not pull %s image from registry, attempting to check changes "
+                "with local version",
+                image_tag,
+            )
+        else:
+            raise
+    try:
+        container = dc.containers.get(dc.api.create_container(image_tag)["Id"])
+    except docker.errors.APIError as e:
+        if e.response.status_code == 404:
             return None
         else:
             raise
-    else:
-        container = dc.containers.get(dc.api.create_container(image_tag)["Id"])
-        try:
-            tarfile_path = tmp_dir / "tar-file.tar.gz"
-            with open(tarfile_path, mode="w+b") as f:
-                try:
-                    stream, _ = dc.api.get_archive(container.id, str(file_path))
-                except docker.errors.NotFound:
-                    pass
-                else:
-                    for chunk in stream:
-                        f.write(chunk)
-                    f.flush()
-        finally:
-            container.remove()
-        with tarfile.open(tarfile_path) as f:
-            f.extractall(out_path)
+    try:
+        tarfile_path = tmp_dir / "tar-file.tar.gz"
+        with open(tarfile_path, mode="w+b") as f:
+            try:
+                stream, _ = dc.api.get_archive(container.id, str(file_path))
+            except docker.errors.NotFound:
+                pass
+            else:
+                for chunk in stream:
+                    f.write(chunk)
+                f.flush()
+    finally:
+        container.remove()
+    with tarfile.open(tarfile_path) as f:
+        f.extractall(out_path)
     return out_path
 
 
@@ -543,6 +600,78 @@ def add_exc_note(e, note):
     return e
 
 
+def dict_diff(dict1, dict2, label1="dict1", label2="dict2"):
+    """Create a human readable diff between two dictionaries
+
+    Parameters
+    ----------
+    dict1 : dict
+        first dictionary to compare
+    dict2 : dict
+        second dictionary to compare
+    label1 : str
+        label to give first dictionary in diff
+    label2 : str
+        label to give second dictionary in diff
+
+    Returns
+    -------
+    diff : str
+        the unified diff between the two dictionaries
+    """
+    yaml1 = yaml.dump(dict1, sort_keys=True, indent=4)
+    yaml2 = yaml.dump(dict2, sort_keys=True, indent=4)
+    diff = difflib.unified_diff(
+        yaml1.splitlines(),
+        yaml2.splitlines(),
+        fromfile=label1,
+        tofile=label2,
+        lineterm="\n",
+    )
+    return "\n".join(diff)
+
+
+def full_path(fspath: ty.Union[str, bytes, Path]):
+    return Path(fspath).resolve().absolute()
+
+
+class fromdict_converter:
+    def __init__(self, tp):
+        try:
+            self.container_type = tp.__origin__
+        except AttributeError:
+            self.type = tp
+            self.container_type = None
+        else:
+            if self.container_type not in (list, tuple, set, frozenset):
+                raise ArcanaError(
+                    f"generic aliases of {self.container_type} type are not supported by "
+                    "fromdict_converter"
+                )
+            self.type = tp.__args__[0]
+
+    def __call__(self, to_convert):
+        if self.container_type:
+            converted = self.container_type(
+                self.type(**d) if isinstance(d, dict) else d for d in to_convert
+            )
+        else:
+            converted = (
+                self.type(**to_convert) if isinstance(to_convert, dict) else to_convert
+            )
+        return converted
+
+
+def show_cli_trace(result):
+    "Used in testing to show traceback of CLI output"
+    return "".join(traceback.format_exception(*result.exc_info))
+
+
+def append_suffix(path: Path, suffix: str) -> Path:
+    "Appends a string suffix to a Path object"
+    return Path(str(path) + suffix)
+
+
 # Minimum version of Arcana that this version can read the serialisation from
 MIN_SERIAL_VERSION = "0.0.0"
 
